# frozen_string_literal: true

#--
# gravaty
# rubocop:disable Style/AsciiComments
# © 2013 Marco Bresciani
# rubocop:enable Style/AsciiComments
#
# This file is part of gravaty.
#
# gravaty is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# gravaty is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with gravaty. If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-FileCopyrightText: 2013 Marco Bresciani
#
# SPDX-License-Identifier: GPL-3.0-or-later
#++

require_relative '../test_helper'

def test_avatar_formats
  Gravaty::AVATAR_FORMATS.each do |avatar|
    describe "when asked for '#{avatar}' image" do
      it "shall provide an URI with specified '#{avatar}' image" do
        _(subject.avatar(type: avatar)).must_include(".#{avatar}")
      end
    end
  end
end

def test_allowed_sizes
  Gravaty::ALLOWED_SIZES.each do |size|
    describe "when asked for a size of '#{size}' pixels" do
      it "shall provide an URI with specified '#{size}' image" do
        _(subject.avatar(pixelsize: size)).must_match(/#{Gravaty::TEST_SIZE_REGEXP}$/)
      end
    end
  end
end

def test_unallowed_sizes
  [-1, 0, Gravaty::ALLOWED_SIZES.max + 1].each do |size|
    describe "when asked for an invalid size of '#{size}' pixels" do
      it 'must raise an argument error' do
        _(-> { subject.avatar(pixelsize: size) }).must_raise ArgumentError
      end
    end
  end
end

def test_rating_options
  Gravaty::RATING_OPTIONS.each do |rating|
    describe "when asked for a specific image rating '#{rating}'" do
      it "shall provide an URI with specified '#{rating}'" do
        _(subject.avatar(rating: rating)).must_include("rating=#{rating}")
      end
    end
  end
end

def test_default_options
  Gravaty::DEFAULT_OPTIONS.each do |options|
    describe "when asked for a specific image options '#{options}'" do
      it "shall provide an URI with option '#{options}'" do
        _(subject.avatar(default: options)).must_include("d=#{options}")
      end
    end
  end
end

def test_own_https_image
  it 'shall provide a HTTP(S) URI' do
    ['', 's'].each do |type|
      an_uri = "http#{type}://www.example.com/example.jpg"
      _(subject.avatar(default: an_uri)).must_include CGI.escape(an_uri)
    end
  end
end

def test_own_default_image
  describe 'when provided with a own default image' do
    test_own_https_image

    it 'must raise an ArgumentError when default parameter is not correct' do
      _(-> { subject.avatar default: 'ftp://www.example.com/example.jpg' }).must_raise ArgumentError
      _(-> { subject.avatar default: 'http://www.example.com/example.svg' }).must_raise ArgumentError
      _(-> { subject.avatar default: 'http://www.example.com?image=example.jpg' }).must_raise ArgumentError
    end
  end
end

def test_unknown_parameters
  describe 'when asked for unknown image' do
    it 'must raise an ArgumentError' do
      _(-> { subject.avatar type: 'unknown' }).must_raise ArgumentError
    end
  end

  describe 'when asked for unknown rating' do
    it 'must raise an ArgumentError' do
      _(-> { subject.avatar rating: 'unknown' }).must_raise ArgumentError
    end
  end
end

describe Gravaty::Gravaty do
  describe 'when created with a valid email address ' do
    subject do
      Gravaty::Gravaty.new(Gravaty::TEST_MY_ADDRESS,
                           Gravaty.parser)
    end

    test_avatar_formats

    test_allowed_sizes

    test_unallowed_sizes

    test_rating_options

    test_unknown_parameters

    describe 'when asked for forcing default image' do
      it 'shall provide a forced URI' do
        _(subject.avatar(force: true)).must_include Gravaty::TEST_FORCED
      end
    end

    test_default_options

    test_own_default_image
  end
end
